@@ -41,12 +41,9 @@ share_opts = [
     cfg.StrOpt('lvm_share_export_root',
                default='$state_path/mnt',
                help='Base folder where exported shares are located.'),
-    cfg.StrOpt('lvm_share_export_ip',
-               deprecated_for_removal=True,
-               deprecated_reason='Use lvm_share_export_ips instead.',
-               help='IP to be added to export string.'),
     cfg.ListOpt('lvm_share_export_ips',
-                help='List of IPs to export shares.'),
+                help='List of IPs to export shares belonging to the LVM '
+                     'storage driver.'),
     cfg.IntOpt('lvm_share_mirrors',
                default=0,
                help='If set, create LVMs with multiple mirrors. Note that '
@@ -78,15 +75,8 @@ class LVMMixin(driver.ExecuteMixin):
                    % self.configuration.lvm_share_volume_group)
             raise exception.InvalidParameterValue(err=msg)
 
-        if (self.configuration.lvm_share_export_ip and
-                self.configuration.lvm_share_export_ips):
-            msg = (_("Only one of lvm_share_export_ip or lvm_share_export_ips"
-                     " may be specified."))
-            raise exception.InvalidParameterValue(err=msg)
-        if not (self.configuration.lvm_share_export_ip or
-                self.configuration.lvm_share_export_ips):
-            msg = (_("Neither lvm_share_export_ip nor lvm_share_export_ips is"
-                     " specified."))
+        if not self.configuration.lvm_share_export_ips:
+            msg = _("The option lvm_share_export_ips must be specified.")
             raise exception.InvalidParameterValue(err=msg)
 
     def _allocate_container(self, share):
@@ -108,7 +98,7 @@ class LVMMixin(driver.ExecuteMixin):
                       device_name, run_as_root=True)
 
     def _extend_container(self, share, device_name, size):
-        cmd = ['lvextend', '-L', '%sG' % size, '-r', device_name]
+        cmd = ['lvextend', '-L', '%sG' % size, '-n', device_name]
         self._try_execute(*cmd, run_as_root=True)
 
     def _deallocate_container(self, share_name):
@@ -118,9 +108,7 @@ class LVMMixin(driver.ExecuteMixin):
                               (self.configuration.lvm_share_volume_group,
                                share_name), run_as_root=True)
         except exception.ProcessExecutionError as exc:
-            err_pattern = re.compile(".*failed to find.*|.*not found.*",
-                                     re.IGNORECASE)
-            if not err_pattern.match(exc.stderr):
+            if "not found" not in exc.stderr:
                 LOG.exception("Error deleting volume")
                 raise
             LOG.warning("Volume not found: %s", exc.stderr)
@@ -178,12 +166,9 @@ class LVMShareDriver(LVMMixin, driver.ShareDriver):
             'instance_id': self.backend_name,
             'lock_name': 'manila_lvm',
         }
-        if self.configuration.lvm_share_export_ip:
-            self.share_server['public_addresses'] = [
-                self.configuration.lvm_share_export_ip]
-        else:
-            self.share_server['public_addresses'] = (
-                self.configuration.lvm_share_export_ips)
+        self.share_server['public_addresses'] = (
+            self.configuration.lvm_share_export_ips
+        )
         self.ipv6_implemented = True
 
     def _ssh_exec_as_root(self, server, command, check_exit_code=True):
@@ -271,11 +256,11 @@ class LVMShareDriver(LVMMixin, driver.ShareDriver):
         return location
 
     def delete_share(self, context, share, share_server=None):
-        self._unmount_device(share, raise_if_missing=False)
+        self._unmount_device(share)
         self._delete_share(context, share)
         self._deallocate_container(share['name'])
 
-    def _unmount_device(self, share_or_snapshot, raise_if_missing=True):
+    def _unmount_device(self, share_or_snapshot):
         """Unmount the filesystem of a share or snapshot LV."""
         mount_path = self._get_mount_path(share_or_snapshot)
         if os.path.exists(mount_path):
@@ -283,13 +268,9 @@ class LVMShareDriver(LVMMixin, driver.ShareDriver):
             try:
                 self._execute('umount', '-f', mount_path, run_as_root=True)
             except exception.ProcessExecutionError as exc:
-                if 'device is busy' in exc.stderr.lower():
+                if 'device is busy' in six.text_type(exc):
                     raise exception.ShareBusyException(
                         reason=share_or_snapshot['name'])
-                elif 'not mounted' in exc.stderr.lower():
-                    if raise_if_missing:
-                        LOG.error('Unable to find device: %s', exc)
-                        raise
                 else:
                     LOG.error('Unable to umount: %s', exc)
                     raise
@@ -398,6 +379,7 @@ class LVMShareDriver(LVMMixin, driver.ShareDriver):
     def extend_share(self, share, new_size, share_server=None):
         device_name = self._get_local_path(share)
         self._extend_container(share, device_name, new_size)
+        self._execute('resize2fs', device_name, run_as_root=True)
 
     def revert_to_snapshot(self, context, snapshot, share_access_rules,
                            snapshot_access_rules, share_server=None):
@@ -448,7 +430,7 @@ class LVMShareDriver(LVMMixin, driver.ShareDriver):
         return {'export_locations': exports}
 
     def delete_snapshot(self, context, snapshot, share_server=None):
-        self._unmount_device(snapshot, raise_if_missing=False)
+        self._unmount_device(snapshot)
 
         super(LVMShareDriver, self).delete_snapshot(context, snapshot,
                                                     share_server)
@@ -457,23 +439,12 @@ class LVMShareDriver(LVMMixin, driver.ShareDriver):
         if self.configured_ip_version is None:
             try:
                 self.configured_ip_version = []
-                if self.configuration.lvm_share_export_ip:
-                    self.configured_ip_version.append(ipaddress.ip_address(
-                        six.text_type(
-                            self.configuration.lvm_share_export_ip)).version)
-                else:
-                    for ip in self.configuration.lvm_share_export_ips:
-                        self.configured_ip_version.append(
-                            ipaddress.ip_address(six.text_type(ip)).version)
+                for ip in self.configuration.lvm_share_export_ips:
+                    self.configured_ip_version.append(
+                        ipaddress.ip_address(six.text_type(ip)).version)
             except Exception:
-                if self.configuration.lvm_share_export_ip:
-                    message = (_("Invalid 'lvm_share_export_ip' option "
-                                 "supplied %s.") %
-                               self.configuration.lvm_share_export_ip)
-                else:
-                    message = (_("Invalid 'lvm_share_export_ips' option "
-                                 "supplied %s.") %
-                               self.configuration.lvm_share_export_ips)
+                message = (_("Invalid 'lvm_share_export_ips' option supplied "
+                             "%s.") % self.configuration.lvm_share_export_ips)
                 raise exception.InvalidInput(reason=message)
         return self.configured_ip_version
 
