@@ -67,9 +67,9 @@ share_servers_handling_mode_opts = [
         help="Security group name, that will be used for "
              "service instance creation. "
              "Only used if driver_handles_share_servers=True."),
-    cfg.IntOpt(
+    cfg.StrOpt(
         "service_instance_flavor_id",
-        default=100,
+        default="100",
         help="ID of flavor, that will be used for service instance "
              "creation. Only used if driver_handles_share_servers=True."),
     cfg.StrOpt(
@@ -93,8 +93,18 @@ share_servers_handling_mode_opts = [
     cfg.StrOpt(
         "interface_driver",
         default="manila.network.linux.interface.OVSInterfaceDriver",
-        help="Vif driver. Used only with Neutron and "
-             "if driver_handles_share_servers=True."),
+        help="Module path to the Virtual Interface (VIF) driver class. This "
+             "option is used only by drivers operating in "
+             "`driver_handles_share_servers=True` mode that provision "
+             "OpenStack compute instances as share servers. This option is "
+             "only supported with Neutron networking. "
+             "Drivers provided in tree work with Linux Bridge "
+             "(manila.network.linux.interface.BridgeInterfaceDriver) and OVS "
+             "(manila.network.linux.interface.OVSInterfaceDriver). If the "
+             "manila-share service is running on a host that is connected to "
+             "the administrator network, a no-op driver "
+             "(manila.network.linux.interface.NoopInterfaceDriver) may "
+             "be used."),
     cfg.BoolOpt(
         "connect_share_server_to_tenant_network",
         default=False,
@@ -973,59 +983,30 @@ class NeutronNetworkHelper(BaseNetworkhelper):
                 'manila-admin-share')
             interface_name = self.vif_driver.get_device_name(port)
             device = ip_lib.IPDevice(interface_name)
-            for fixed_ip in port['fixed_ips']:
-                subnet = self.neutron_api.get_subnet(fixed_ip['subnet_id'])
-                device.route.clear_outdated_routes(subnet['cidr'])
-            self._plug_interface_in_host(interface_name, device, port)
+            self._plug_interface_in_host(interface_name, device, port,
+                                         clear_outdated_routes=True)
 
     @utils.synchronized("service_instance_plug_interface_in_host",
                         external=True)
-    def _plug_interface_in_host(self, interface_name, device, port):
+    def _plug_interface_in_host(self, interface_name, device, port,
+                                clear_outdated_routes=False):
 
         LOG.debug("Plug interface into host - interface_name: %s, "
                   "device: %s, port: %s", interface_name, device, port)
         self.vif_driver.plug(interface_name, port['id'], port['mac_address'])
+        cidrs_to_clear = []
         ip_cidrs = []
         for fixed_ip in port['fixed_ips']:
             subnet = self.neutron_api.get_subnet(fixed_ip['subnet_id'])
+            if clear_outdated_routes:
+                cidrs_to_clear.append(subnet['cidr'])
+
             net = netaddr.IPNetwork(subnet['cidr'])
             ip_cidr = '%s/%s' % (fixed_ip['ip_address'], net.prefixlen)
             ip_cidrs.append(ip_cidr)
 
-        self.vif_driver.init_l3(interface_name, ip_cidrs)
-
-        # ensure that interface is first in the list
-        device.route.pullup_route(interface_name)
-
-        # here we are checking for garbage devices from removed service port
-        self._remove_outdated_interfaces(device)
-
-    def _remove_outdated_interfaces(self, device):
-        """Finds and removes unused network device."""
-        device_cidr_set = self._get_set_of_device_cidrs(device)
-        for dev in ip_lib.IPWrapper().get_devices():
-            if dev.name != device.name and dev.name[:3] == device.name[:3]:
-                cidr_set = self._get_set_of_device_cidrs(dev)
-                if device_cidr_set & cidr_set:
-                    self.vif_driver.unplug(dev.name)
-
-    def _get_set_of_device_cidrs(self, device):
-        cidrs = set()
-        addr_list = []
-        try:
-            # NOTE(ganso): I could call ip_lib.device_exists here, but since
-            # this is a concurrency problem, it would not fix the problem.
-            addr_list = device.addr.list()
-        except Exception as e:
-            if 'does not exist' in six.text_type(e):
-                LOG.warning(
-                    "Device %s does not exist anymore.", device.name)
-            else:
-                raise
-        for addr in addr_list:
-            if addr['ip_version'] == 4:
-                cidrs.add(six.text_type(netaddr.IPNetwork(addr['cidr']).cidr))
-        return cidrs
+        self.vif_driver.init_l3(interface_name, ip_cidrs,
+                                clear_cidrs=cidrs_to_clear)
 
     @utils.synchronized("service_instance_get_service_port", external=True)
     def _get_service_port(self, network_id, subnet_id, device_id):
