@@ -30,6 +30,7 @@ from manila import db
 from manila import exception
 from manila.scheduler.filters import base_host
 from manila.scheduler import host_manager
+from manila.scheduler import utils as scheduler_utils
 from manila import test
 from manila.tests.scheduler import fakes
 from manila import utils
@@ -589,8 +590,10 @@ class HostManagerTestCase(test.TestCase):
                     },
                 },
             ]
+            self.assertTrue(isinstance(res, list))
             self.assertEqual(len(expected), len(res))
-            self.assertEqual(sorted(expected), sorted(res))
+            for pool in expected:
+                self.assertIn(pool, res)
 
     @ddt.data(
         None,
@@ -791,10 +794,13 @@ class HostStateTestCase(test.TestCase):
         fake_context = context.RequestContext('user', 'project', is_admin=True)
         share_size = 10
         free_capacity = 100
+        provisioned_capacity_gb = 50
         fake_share = {'id': 'foo', 'size': share_size}
         share_capability = {
             'total_capacity_gb': free_capacity * 2,
             'free_capacity_gb': free_capacity,
+            'provisioned_capacity_gb': provisioned_capacity_gb,
+            'allocated_capacity_gb': provisioned_capacity_gb,
             'reserved_percentage': 0,
             'timestamp': None
         }
@@ -805,11 +811,17 @@ class HostStateTestCase(test.TestCase):
         fake_host.consume_from_share(fake_share)
         self.assertEqual(fake_host.free_capacity_gb,
                          free_capacity - share_size)
+        self.assertEqual(fake_host.provisioned_capacity_gb,
+                         provisioned_capacity_gb + share_size)
+        self.assertEqual(fake_host.allocated_capacity_gb,
+                         provisioned_capacity_gb + share_size)
 
     def test_consume_from_share_unknown_capability(self):
         share_capability = {
             'total_capacity_gb': 'unknown',
             'free_capacity_gb': 'unknown',
+            'provisioned_capacity_gb': None,
+            'allocated_capacity_gb': 0,
             'reserved_percentage': 0,
             'timestamp': None
         }
@@ -823,13 +835,18 @@ class HostStateTestCase(test.TestCase):
         fake_host.consume_from_share(fake_share)
         self.assertEqual(fake_host.total_capacity_gb, 'unknown')
         self.assertEqual(fake_host.free_capacity_gb, 'unknown')
+        self.assertIsNone(fake_host.provisioned_capacity_gb)
+        self.assertEqual(fake_host.allocated_capacity_gb, share_size)
 
     def test_consume_from_share_invalid_capacity(self):
         fake_host = host_manager.PoolState('host1', {}, '_pool0')
         fake_host.free_capacity_gb = 'invalid_foo_string'
+        fake_host.provisioned_capacity_gb = None
+        fake_host.allocated_capacity_gb = 0
+        fake_share = {'id': 'fake', 'size': 10}
 
         self.assertRaises(exception.InvalidCapacity,
-                          fake_host.consume_from_share, 'fake')
+                          fake_host.consume_from_share, fake_share)
 
     def test_repr(self):
 
@@ -865,6 +882,7 @@ class PoolStateTestCase(test.TestCase):
             'share_capability':
                 {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
                  'reserved_percentage': 0, 'timestamp': None,
+                 'thin_provisioning': True,
                  'cap1': 'val1', 'cap2': 'val2'},
             'instances':
                 [
@@ -882,6 +900,70 @@ class PoolStateTestCase(test.TestCase):
                     },
                 ]
         },
+        {
+            'share_capability':
+                {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
+                 'reserved_percentage': 0, 'timestamp': None,
+                 'thin_provisioning': False, 'cap1': 'val1', 'cap2': 'val2'},
+            'instances':
+                [
+                    {
+                        'id': 1, 'host': 'host1',
+                        'status': 'available',
+                        'share_id': 11, 'size': 1,
+                        'updated_at': timeutils.utcnow()
+                    },
+                    {
+                        'id': 2, 'host': 'host1',
+                        'status': 'available',
+                        'share_id': 12, 'size': None,
+                        'updated_at': timeutils.utcnow()
+                    },
+                ]
+        },
+        {
+            'share_capability':
+                {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
+                 'reserved_percentage': 0, 'timestamp': None,
+                 'thin_provisioning': [False], 'cap1': 'val1', 'cap2': 'val2'},
+            'instances':
+                [
+                    {
+                        'id': 1, 'host': 'host1',
+                        'status': 'available',
+                        'share_id': 11, 'size': 1,
+                        'updated_at': timeutils.utcnow()
+                    },
+                    {
+                        'id': 2, 'host': 'host1',
+                        'status': 'available',
+                        'share_id': 12, 'size': None,
+                        'updated_at': timeutils.utcnow()
+                    },
+                ]
+        },
+        {
+            'share_capability':
+                {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
+                 'reserved_percentage': 0, 'timestamp': None,
+                 'thin_provisioning': [True, False], 'cap1': 'val1',
+                 'cap2': 'val2'},
+            'instances':
+                [
+                    {
+                        'id': 1, 'host': 'host1',
+                        'status': 'available',
+                        'share_id': 11, 'size': 4,
+                        'updated_at': timeutils.utcnow()
+                    },
+                    {
+                        'id': 2, 'host': 'host1',
+                        'status': 'available',
+                        'share_id': 12, 'size': None,
+                        'updated_at': timeutils.utcnow()
+                    },
+                ]
+        },
         {
             'share_capability':
                 {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
@@ -890,6 +972,30 @@ class PoolStateTestCase(test.TestCase):
                  'ipv6_support': False},
             'instances': []
         },
+        {
+            'share_capability':
+                {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
+                 'reserved_percentage': 0, 'timestamp': None,
+                 'thin_provisioning': True, 'cap1': 'val1', 'cap2': 'val2',
+                 'ipv4_support': True, 'ipv6_support': False},
+            'instances': []
+        },
+        {
+            'share_capability':
+                {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
+                 'reserved_percentage': 0, 'timestamp': None,
+                 'thin_provisioning': [False], 'cap1': 'val1', 'cap2': 'val2',
+                 'ipv4_support': True, 'ipv6_support': False},
+            'instances': []
+        },
+        {
+            'share_capability':
+                {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
+                 'reserved_percentage': 0, 'timestamp': None,
+                 'thin_provisioning': [True, False], 'cap1': 'val1',
+                 'cap2': 'val2', 'ipv4_support': True, 'ipv6_support': False},
+            'instances': []
+        },
         {
             'share_capability':
                 {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
@@ -932,12 +1038,76 @@ class PoolStateTestCase(test.TestCase):
                     },
                 ]
         },
+        {
+            'share_capability':
+                {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
+                 'allocated_capacity_gb': 256, 'provisioned_capacity_gb': 1,
+                 'thin_provisioning': True, 'reserved_percentage': 0,
+                 'timestamp': None, 'cap1': 'val1', 'cap2': 'val2'},
+            'instances':
+                [
+                    {
+                        'id': 1, 'host': 'host1',
+                        'status': 'available',
+                        'share_id': 11, 'size': 1,
+                        'updated_at': timeutils.utcnow()
+                    },
+                ]
+        },
+        {
+            'share_capability':
+                {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
+                 'allocated_capacity_gb': 256, 'provisioned_capacity_gb': 1,
+                 'thin_provisioning': [False], 'reserved_percentage': 0,
+                 'timestamp': None, 'cap1': 'val1', 'cap2': 'val2'},
+            'instances':
+                [
+                    {
+                        'id': 1, 'host': 'host1',
+                        'status': 'available',
+                        'share_id': 11, 'size': 1,
+                        'updated_at': timeutils.utcnow()
+                    },
+                ]
+        },
+        {
+            'share_capability':
+                {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
+                 'allocated_capacity_gb': 256, 'provisioned_capacity_gb': 1,
+                 'thin_provisioning': [True, False], 'reserved_percentage': 0,
+                 'timestamp': None, 'cap1': 'val1', 'cap2': 'val2'},
+            'instances':
+                [
+                    {
+                        'id': 1, 'host': 'host1',
+                        'status': 'available',
+                        'share_id': 11, 'size': 1,
+                        'updated_at': timeutils.utcnow()
+                    },
+                ]
+        },
         {
             'share_capability':
                 {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
                  'allocated_capacity_gb': 256, 'provisioned_capacity_gb': 256,
-                 'reserved_percentage': 0, 'timestamp': None, 'cap1': 'val1',
-                 'cap2': 'val2'},
+                 'thin_provisioning': False, 'reserved_percentage': 0,
+                 'timestamp': None, 'cap1': 'val1', 'cap2': 'val2'},
+            'instances':
+                [
+                    {
+                        'id': 1, 'host': 'host1',
+                        'status': 'available',
+                        'share_id': 11, 'size': 1,
+                        'updated_at': timeutils.utcnow()
+                    },
+                ]
+        },
+        {
+            'share_capability':
+                {'total_capacity_gb': 1024, 'free_capacity_gb': 512,
+                 'allocated_capacity_gb': 256, 'provisioned_capacity_gb': 256,
+                 'thin_provisioning': [False], 'reserved_percentage': 0,
+                 'timestamp': None, 'cap1': 'val1', 'cap2': 'val2'},
             'instances':
                 [
                     {
@@ -967,35 +1137,42 @@ class PoolStateTestCase(test.TestCase):
         self.assertEqual(512, fake_pool.free_capacity_gb)
         self.assertDictMatch(share_capability, fake_pool.capabilities)
 
-        if 'provisioned_capacity_gb' not in share_capability:
-            db.share_instances_get_all_by_host.assert_called_once_with(
-                fake_context, fake_pool.host, with_share_data=True)
+        if 'thin_provisioning' in share_capability:
+            thin_provisioned = scheduler_utils.thin_provisioning(
+                share_capability['thin_provisioning'])
+        else:
+            thin_provisioned = False
 
-            if len(instances) > 0:
-                self.assertEqual(4, fake_pool.provisioned_capacity_gb)
+        if thin_provisioned:
+            self.assertEqual(thin_provisioned, fake_pool.thin_provisioning)
+            if 'provisioned_capacity_gb' not in share_capability or (
+                    share_capability['provisioned_capacity_gb'] is None):
+                db.share_instances_get_all_by_host.assert_called_once_with(
+                    fake_context, fake_pool.host, with_share_data=True)
+                if len(instances) > 0:
+                    self.assertEqual(4, fake_pool.provisioned_capacity_gb)
+                else:
+                    self.assertEqual(0, fake_pool.provisioned_capacity_gb)
             else:
-                self.assertEqual(0, fake_pool.provisioned_capacity_gb)
-
-            if 'allocated_capacity_gb' in share_capability:
-                self.assertEqual(share_capability['allocated_capacity_gb'],
-                                 fake_pool.allocated_capacity_gb)
-            elif 'allocated_capacity_gb' not in share_capability:
-                self.assertEqual(0, fake_pool.allocated_capacity_gb)
-        elif 'provisioned_capacity_gb' in share_capability and (
-                'allocated_capacity_gb' not in share_capability):
-            self.assertFalse(db.share_instances_get_all_by_host.called)
-
-            self.assertEqual(0, fake_pool.allocated_capacity_gb)
-            self.assertEqual(share_capability['provisioned_capacity_gb'],
-                             fake_pool.provisioned_capacity_gb)
-        elif 'provisioned_capacity_gb' in share_capability and (
-                'allocated_capacity_gb' in share_capability):
+                self.assertFalse(db.share_instances_get_all_by_host.called)
+                self.assertEqual(share_capability['provisioned_capacity_gb'],
+                                 fake_pool.provisioned_capacity_gb)
+        else:
+            self.assertFalse(fake_pool.thin_provisioning)
             self.assertFalse(db.share_instances_get_all_by_host.called)
+            if 'provisioned_capacity_gb' not in share_capability or (
+                    share_capability['provisioned_capacity_gb'] is None):
+                self.assertIsNone(fake_pool.provisioned_capacity_gb)
+            else:
+                self.assertEqual(share_capability['provisioned_capacity_gb'],
+                                 fake_pool.provisioned_capacity_gb)
 
+        if 'allocated_capacity_gb' in share_capability:
             self.assertEqual(share_capability['allocated_capacity_gb'],
                              fake_pool.allocated_capacity_gb)
-            self.assertEqual(share_capability['provisioned_capacity_gb'],
-                             fake_pool.provisioned_capacity_gb)
+        else:
+            self.assertEqual(0, fake_pool.allocated_capacity_gb)
+
         if 'ipv4_support' in share_capability:
             self.assertEqual(share_capability['ipv4_support'],
                              fake_pool.ipv4_support)
